import os
import logging
from pathlib import Path
from typing import Optional, List

from .process_controller import start_process

default_logger = logging.getLogger(__name__)

current_file_path = os.path.dirname(os.path.realpath(__file__))

java_binary_name = "java.exe" if os.name == "nt" else "java"


def find_java_bin(extra_location: str = None):
    locations = [
        Path(current_file_path, f"../../../../gateway/jre/bin/{java_binary_name}"),
        "/opt/dynatrace/gateway/jre/bin/java",
        "C:/Program Files/dynatrace/gateway/jre/bin/java.exe",
        "D:/Program Files/dynatrace/gateway/jre/bin/java.exe",
    ]
    if extra_location:
        locations.insert(0, extra_location)

    for location in locations:
        path = Path(location).resolve()
        default_logger.info(f"Looking for java under '{path}'")
        if path.exists():
            return str(path)

    if os.environ.get("GATEWAY_HOME", False):
        return f"{os.environ.get('GATEWAY_HOME')}/jre/bin/java"

    return "java"


def query(endpoint_id: int, temp_folder: Path, env=None, java_path: Optional[str] = None, jvm_extra_args: Optional[List[str]] = None) -> None:
    parent = Path(current_file_path).parent

    # Resolve with * breaks on windows https://bugs.python.org/issue35306
    jar_path = Path(Path(parent, "jars").resolve(), "*")
    cmd_args = [
        "-cp",
        f"{jar_path}",
        "com.dynatrace.jdbc.connector.Main",
        "-endpoint",
        f"{endpoint_id}",
        "-tempfolder",
        f"{temp_folder}",
    ]
    if jvm_extra_args is not None:
        cmd_args[0:0] = jvm_extra_args

    java_bin = find_java_bin(java_path)
    cmd = [java_bin, *cmd_args]
    start_process(cmd, env=env)
